package schelling;

import java.util.*;
/**
 * @author rholguin
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
public class Simulation {
	
    private String[] newGrid;  //hold the new updated grid
	private String[] origGrid;  //hold the original grid
	private int[] beforeHappyLevel = {0,0};  //hold before happy levels
	private int[] afterHappyLevel = {0,0};  //hold after happy levels
	private Random ran;
	private int redTol;  //red tolerance level
	private int greenTol;  //green tolerance level
	private int redCount = 0;  //num of red cells
	private int greenCount = 0;  //num of green cells
	
	private int redMoves;  
	private int greenMoves; 
	private int greenHappy;
	private int redHappy;
	
	private static final char EMPTY = '0';
	private static final char GREEN = '1';
	private static final char RED = '2';
	
	/*
	 * This is the constructor that will initialize the variables
	 */
	public Simulation(String[] g, int red, int green) {
		origGrid = g;  
		newGrid = (String[])origGrid.clone();
		
		redTol = red;
		greenTol = green;
		beforeHappyLevel = setHappyLevel();	
		
		ran = new Random();
	}
	
	/*
	 * Initialze the simulation to begin processing
	 */
	public void initSim(int red, int green) {
		redTol = red;
		greenTol = green;
		newGrid = (String[])origGrid.clone();
		
		redMoves = 0;
		greenMoves = 0;
	}
	
	/*
	 * This will run the simulation by choosing a grid space at random and evaluating its status
	 */
	public void run () {
		int ranX = ran.nextInt(newGrid.length);  //get random X coordinate
		int ranY = ran.nextInt(newGrid.length);  //get random Y coordinate
	
		/*
		 * Evaluate the grid space to see if that is a green or red space.  it will check its neighbors
		 * and decide what the action to take is.  if the ratio value is less then its tolerance it will go to
		 * the move functions
		 */
		if(newGrid[ranX].charAt(ranY) == GREEN) {
			int max = checkTolerance(ranX,ranY);
			int temp = (redCount*100) / max;
			
			if(temp >= greenTol) {
				move(ranX,ranY,GREEN);
			}
		}
		else if (newGrid[ranX].charAt(ranY) == RED) {
			
			int max = checkTolerance(ranX,ranY);
			int temp = (greenCount*100) / max;
		
			if(temp >= redTol) {
				move(ranX,ranY,RED);
			}
		}
		
		afterHappyLevel = setHappyLevel();  //set the happy levels for the grid
	}
	
	/*
	 * This method will take the new grid and set the happy levels of all teh grid squres
	 */
	private int[] setHappyLevel() {
		greenHappy = 0;
		redHappy = 0;
		for(int x = 0; x < newGrid.length; x++) {
			for(int y = 0; y < newGrid.length; y++) {
				if(newGrid[x].charAt(y) == GREEN) {
					int max = checkTolerance(x,y);
					int temp = (redCount*100) / max;
					
					if(temp <= greenTol) {
						greenHappy++;
					}
				}
				else if (newGrid[x].charAt(y) == RED) {
					
					int max = checkTolerance(x,y);
					int temp = (greenCount*100) / max;
				
					if(temp <= redTol) {
						redHappy++;
					}
				}
			}
		}
		int [] happyLevel= {greenHappy, redHappy};  //set happy level for red and green
		return happyLevel;
	}
	
	public int [] getBeforeHappyLevel() {
		return beforeHappyLevel;
	}
	
	public int [] getAfterHappyLevel() {
		return afterHappyLevel;
	}
	
	/*
	 * This will check the neighbors of the current cell.  depending on what color the neighbor is the
	 * color count will increase by one
	 */
	private int checkTolerance(int x, int y) {
		redCount = 0;
		greenCount = 0;
		
		//if its a corner cell
		if(x == 0 && y == 0) {
			if(newGrid[x].charAt(y+1) == GREEN) greenCount++; else if(newGrid[x].charAt(y+1) == RED) redCount++;
			if(newGrid[x+1].charAt(y+1) == GREEN) greenCount++; else if(newGrid[x+1].charAt(y+1) == RED) redCount++;
			if(newGrid[x+1].charAt(y) == GREEN) greenCount++; else if(newGrid[x+1].charAt(y) == RED) redCount++;
			return 3;
		}
		else if(x == newGrid.length-1 && y == 0) {
			if(newGrid[x].charAt(y+1) == GREEN) greenCount++; else if(newGrid[x].charAt(y+1) == RED) redCount++;
			if(newGrid[x-1].charAt(y+1) == GREEN) greenCount++; else if(newGrid[x-1].charAt(y+1) == RED) redCount++;
			if(newGrid[x-1].charAt(y) == GREEN) greenCount++; else if(newGrid[x-1].charAt(y) == RED) redCount++;
			return 3;
		}
		else if(x == 0 && y == newGrid.length-1) {
			if(newGrid[x].charAt(y-1) == GREEN) greenCount++; else if(newGrid[x].charAt(y-1) == RED) redCount++;
			if(newGrid[x+1].charAt(y-1) == GREEN) greenCount++; else if(newGrid[x+1].charAt(y-1) == RED) redCount++;
			if(newGrid[x+1].charAt(y) == GREEN) greenCount++; else if(newGrid[x+1].charAt(y) == RED) redCount++;
			return 3;
		}
		else if(x == newGrid.length-1 && y == newGrid.length-1) {
			if(newGrid[x].charAt(y-1) == GREEN) greenCount++; else if(newGrid[x].charAt(y-1) == RED) redCount++;
			if(newGrid[x-1].charAt(y-1) == GREEN) greenCount++; else if(newGrid[x-1].charAt(y-1) == RED) redCount++;
			if(newGrid[x-1].charAt(y) == GREEN) greenCount++; else if(newGrid[x-1].charAt(y) == RED) redCount++;
			return 3;
		}
		//if its an edge cell
		else if(y == 0) {
			if(newGrid[x-1].charAt(y) == GREEN) greenCount++; else if(newGrid[x-1].charAt(y) == RED) redCount++;
			if(newGrid[x-1].charAt(y+1) == GREEN) greenCount++; else if(newGrid[x-1].charAt(y+1) == RED) redCount++;
			if(newGrid[x].charAt(y+1) == GREEN) greenCount++; else if(newGrid[x].charAt(y+1) == RED) redCount++;
			if(newGrid[x+1].charAt(y+1) == GREEN) greenCount++; else if(newGrid[x+1].charAt(y+1) == RED) redCount++;
			if(newGrid[x+1].charAt(y) == GREEN) greenCount++; else if(newGrid[x+1].charAt(y) == RED) redCount++;
			return 5;
		}
		else if(y == newGrid.length-1) {
			if(newGrid[x-1].charAt(y) == GREEN) greenCount++; else if(newGrid[x-1].charAt(y) == RED) redCount++;
			if(newGrid[x-1].charAt(y-1) == GREEN) greenCount++; else if(newGrid[x-1].charAt(y-1) == RED) redCount++;
			if(newGrid[x].charAt(y-1) == GREEN) greenCount++; else if(newGrid[x].charAt(y-1) == RED) redCount++;
			if(newGrid[x+1].charAt(y-1) == GREEN) greenCount++; else if(newGrid[x+1].charAt(y-1) == RED) redCount++;
			if(newGrid[x+1].charAt(y) == GREEN) greenCount++; else if(newGrid[x+1].charAt(y) == RED) redCount++;
			return 5;
		}
		else if(x == 0) {
			if(newGrid[x].charAt(y-1) == GREEN) greenCount++; else if(newGrid[x].charAt(y-1) == RED) redCount++;
			if(newGrid[x+1].charAt(y-1) == GREEN) greenCount++; else if(newGrid[x+1].charAt(y-1) == RED) redCount++;
			if(newGrid[x+1].charAt(y) == GREEN) greenCount++; else if(newGrid[x+1].charAt(y) == RED) redCount++;
			if(newGrid[x+1].charAt(y+1) == GREEN) greenCount++; else if(newGrid[x+1].charAt(y+1) == RED) redCount++;
			if(newGrid[x].charAt(y+1) == GREEN) greenCount++; else if(newGrid[x].charAt(y+1) == RED) redCount++;
			return 5;
		}
		else if(x == newGrid.length-1) {
			if(newGrid[x].charAt(y-1) == GREEN) greenCount++; else if(newGrid[x].charAt(y-1) == RED) redCount++;
			if(newGrid[x-1].charAt(y-1) == GREEN) greenCount++; else if(newGrid[x-1].charAt(y-1) == RED) redCount++;
			if(newGrid[x-1].charAt(y) == GREEN) greenCount++; else if(newGrid[x-1].charAt(y) == RED) redCount++;
			if(newGrid[x-1].charAt(y+1) == GREEN) greenCount++; else if(newGrid[x-1].charAt(y+1) == RED) redCount++;
			if(newGrid[x].charAt(y+1) == GREEN) greenCount++; else if(newGrid[x].charAt(y+1) == RED) redCount++;
			return 5;
		}
		//if the cell is not edge or corner cell
		else {
			if(newGrid[x+1].charAt(y) == GREEN) greenCount++; else if(newGrid[x+1].charAt(y) == RED) redCount++;
			if(newGrid[x-1].charAt(y) == GREEN) greenCount++; else if(newGrid[x-1].charAt(y) == RED) redCount++;
			if(newGrid[x].charAt(y+1) == GREEN) greenCount++; else if(newGrid[x].charAt(y+1) == RED) redCount++;
			if(newGrid[x].charAt(y-1) == GREEN) greenCount++; else if(newGrid[x].charAt(y-1) == RED) redCount++;
			if(newGrid[x+1].charAt(y+1) == GREEN) greenCount++; else if(newGrid[x+1].charAt(y+1) == RED) redCount++;
			if(newGrid[x+1].charAt(y-1) == GREEN) greenCount++; else if(newGrid[x+1].charAt(y-1) == RED) redCount++;
			if(newGrid[x-1].charAt(y+1) == GREEN) greenCount++; else if(newGrid[x-1].charAt(y+1) == RED) redCount++;
			if(newGrid[x-1].charAt(y-1) == GREEN) greenCount++; else if(newGrid[x-1].charAt(y-1) == RED) redCount++;
			return 8;
		}
	}
	
	/*
	 * This will move the given cell to one of the adjacent cells if there is an empty space
	 */
	private void move(int x, int y, char type) {
		String temp = "";  //will hold a single row of the grid
			
		//if there is an empty space to move too 
		if(getMoveCoordinates(x,y) != null) {
		    
		    //check the type and increment the moves counter
			if(type == GREEN) {
				greenMoves++;
			}
			else
			{
				redMoves++;
			}
			
			//set x and y coordinates
			int newX = new Integer(getMoveCoordinates(x,y).charAt(0)).intValue() - 48;
			int newY = new Integer(getMoveCoordinates(x,y).charAt(1)).intValue() - 48;
			
			/*
		     * this finds the row that the empty cell belongs too and iteratres through all the cell in that line.
		     * when the empty cell is reached is set to the value of the cell that is moving.
		     * These values are added to a string and in the end the row w/ the updated value is set on the grid
		     */
			for(int z = 0; z < newGrid[newX].length(); z++) {
				if(z == newY) {
					temp += type;
				}
				else {
					temp += String.valueOf(newGrid[newX].charAt(z));
				}
			}
			newGrid[newX] = temp;
			
			temp = "";
			
			/*
			 * same as above except this loop will replace the moving grid space with an empty value
			 */
			for(int z = 0; z < newGrid[x].length(); z++) {
				if(z == y) {
					temp += EMPTY;
				}
				else {
					temp += String.valueOf(newGrid[x].charAt(z));
				}
			}
			newGrid[x] = temp;
		}
	}
	
	/*
	 * This will find the first empty neighbor and return its coordinates, if there is not empty neighbor to move to
	 * then return null and do no move that cell
	 */
	public String getMoveCoordinates(int x, int y) {
		if(x == 0 && y == 0) {
			if(newGrid[x].charAt(y+1) == EMPTY) return x+""+(y+1);
			if(newGrid[x+1].charAt(y+1) == EMPTY) return (x+1)+""+(y+1);
			if(newGrid[x+1].charAt(y) == EMPTY) return (x+1)+""+y;
			return null;
		}
		else if(x == newGrid.length-1 && y == 0) {
			if(newGrid[x].charAt(y+1) == EMPTY) return x+""+y+1;
			if(newGrid[x-1].charAt(y+1) == EMPTY) return (x-1)+""+(y+1);
			if(newGrid[x-1].charAt(y) == EMPTY) return (x-1)+""+y;
			return null;
		}
		else if(x == 0 && y == newGrid.length-1) {
			if(newGrid[x].charAt(y-1) == EMPTY) return x+""+(y-1);
			if(newGrid[x+1].charAt(y-1) == EMPTY) return (x+1)+""+(y-1);
			if(newGrid[x+1].charAt(y) == EMPTY) return (x+1)+""+y;
			return null;
		}
		else if(x == newGrid.length-1 && y == newGrid.length-1) {
			if(newGrid[x].charAt(y-1) == EMPTY) return x+""+(y-1);
			if(newGrid[x-1].charAt(y-1) == EMPTY) return (x-1)+""+(y-1); 
			if(newGrid[x-1].charAt(y) == EMPTY) return (x-1)+""+y;
			return null;
		}
		else if(y == 0) {
			if(newGrid[x-1].charAt(y) == EMPTY) return (x-1)+""+y;
			if(newGrid[x-1].charAt(y+1) == EMPTY) return (x-1)+""+(y+1);
			if(newGrid[x].charAt(y+1) == EMPTY) return x+""+(y+1);
			if(newGrid[x+1].charAt(y+1) == EMPTY) return (x+1)+""+(y+1);
			if(newGrid[x+1].charAt(y) == EMPTY) return (x+1)+""+y;
			return null;
		}
		else if(y == newGrid.length-1) {
			if(newGrid[x-1].charAt(y) == EMPTY) return (x-1)+""+y;
			if(newGrid[x-1].charAt(y-1) == EMPTY) return (x-1)+""+(y-1); 
			if(newGrid[x].charAt(y-1) == EMPTY) return x+""+(y-1);
			if(newGrid[x+1].charAt(y-1) == EMPTY) return (x+1)+""+(y-1);
			if(newGrid[x+1].charAt(y) == EMPTY) return (x+1)+""+y;
			return null;
		}
		else if(x == 0) {
			if(newGrid[x].charAt(y-1) == EMPTY) return x+""+(y-1); 
			if(newGrid[x+1].charAt(y-1) == EMPTY) return (x+1)+""+(y-1);
			if(newGrid[x+1].charAt(y) == EMPTY) return (x+1)+""+y;
			if(newGrid[x+1].charAt(y+1) == EMPTY) return (x+1)+""+(y+1);
			if(newGrid[x].charAt(y+1) == EMPTY) return x+""+(y+1);
			return null;
		}
		else if(x == newGrid.length-1) {
			if(newGrid[x].charAt(y-1) == EMPTY) return x+""+(y-1);
			if(newGrid[x-1].charAt(y-1) == EMPTY) return (x-1)+""+(y-1);
			if(newGrid[x-1].charAt(y) == EMPTY) return (x-1)+""+y;
			if(newGrid[x-1].charAt(y+1) == EMPTY) return (x-1)+""+(y+1);
			if(newGrid[x].charAt(y+1) == EMPTY) return x+""+(y+1);
			return null;
		}
		else {
			if(newGrid[x+1].charAt(y) == EMPTY) return (x+1)+""+y;
			if(newGrid[x-1].charAt(y) == EMPTY) return (x-1)+""+y;
			if(newGrid[x].charAt(y+1) == EMPTY) return x+""+(y+1);
			if(newGrid[x].charAt(y-1) == EMPTY) return x+""+(y-1);
			if(newGrid[x+1].charAt(y+1) == EMPTY) return (x+1)+""+(y+1);
			if(newGrid[x+1].charAt(y-1) == EMPTY) return (x+1)+""+(y-1);
			if(newGrid[x-1].charAt(y+1) == EMPTY) return (x-1)+""+(y+1); 
			if(newGrid[x-1].charAt(y-1) == EMPTY) return (x-1)+""+(y-1);
			return null;
		}
	}
	
	public int getRedMoves() {
		return redMoves;
	}
	
	public int getGreenMoves() {
		return greenMoves;
	}
	
	public String[] getNewGrid() {
		return newGrid;
	}
}
